﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Interactivity;
using System.Windows.Media;
using System.Windows.Media.Animation;

namespace PerformanceDemo
{
    class AnimateBehavior : Behavior<Grid>
    {
        public string Property { get; set; }

        public ColorAnimation ColorAnimation { get; set; }
        public Brush BackgroundBrush { get; set; }
        protected override void OnAttached()
        {
            BackgroundBrush = new SolidColorBrush { Color = Colors.Transparent };
            ColorAnimation = new ColorAnimation
                {
                    From = Colors.Transparent,
                    To = Colors.Red,
                    Duration = new Duration(TimeSpan.FromMilliseconds(400)),
                    AutoReverse = true,
                    FillBehavior = FillBehavior.Stop
                };

            AssociatedObject.Background = BackgroundBrush;
            this.AssociatedObject.DataContextChanged += AssociatedObject_DataContextChanged; 
            this.WireEvents();
        }

        void AssociatedObject_DataContextChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            BackgroundBrush = new SolidColorBrush { Color = Colors.Transparent };
            if (e.OldValue != null)
            {
                var olddata = e.OldValue as INotifyPropertyChanged;
                if (olddata != null)
                    olddata.PropertyChanged -= data_PropertyChanged;
            }
            this.WireEvents();
        }

        void WireEvents()
        {
            var data = this.AssociatedObject.DataContext as INotifyPropertyChanged;
            if (data != null)
                data.PropertyChanged += data_PropertyChanged;
        }

        void data_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName.Equals("Change"))
            {
                AssociatedObject.Background.BeginAnimation(SolidColorBrush.ColorProperty, ColorAnimation);
                //BackgroundBrush.BeginAnimation(SolidColorBrush.ColorProperty, ColorAnimation);
            }
        }

        protected override void OnDetaching()
        {
            if (this.AssociatedObject.DataContext == null) return;

            var data = this.AssociatedObject.DataContext as INotifyPropertyChanged;
            if (data != null)
                data.PropertyChanged -= data_PropertyChanged;
        }
    }
}
